package
{
	import flash.display.Bitmap;
	import flash.display.Shape;
	import flash.geom.Point;
	import flash.ui.Mouse;
	
	import org.flixel.*;
	import org.flixel.FlxSprite;
	import org.flixel.FlxState;
	import org.flixel.FlxU;
	
	public class GameState extends FlxState
	{
		[Embed(source = "data/mockup.png")] public static var ImgBG:Class;
		[Embed(source = "data/jumper.png")] public static var ImgPlayer:Class;
		[Embed(source = "data/poles.png")] 	public static var ImgPole:Class;
		[Embed(source = "data/walking.mp3")] public static var SndWalk:Class;
		[Embed(source = "data/running.mp3")] public static var SndRun:Class;
		[Embed(source = "data/climbing.mp3")] public static var SndClimb:Class;
		[Embed(source = "data/jumping.mp3")] public static var SndJump:Class;
		
		public var player:FlxSprite;
		public var playerstate:int;
		
		public const heightBefore:Number=415;
		public const heightAfter:Number=465;
		public const polePosition:Number=216;
		public const dropPosition:Number=203;
		public const playerHeight:Number=16;
		public const playerStartL:Number=16;
		
		public var lengthDisplay:FlxText;
		public var restartDisplay:FlxText;
		
		public var rotspd:Number;
		public var pos:Number;
		public var angle:Number;
		public var pole:FlxSprite;
		
		override public function create():void
		{		
			rotspd=0;
			_time=0;
			pos=0;
			angle=0;
			playerstate=0;
			super.create();	
			
			var bg:FlxSprite = new FlxSprite(0,0,ImgBG);
			add(bg);
			
			var instructions:FlxText = new FlxText(71,110,FlxG.width,"tap x to run or climb");
			instructions.size = 16;
			instructions.color = 0xFFEEEEEE;
			add(instructions);
			
			var instructions2:FlxText = new FlxText(50,140,FlxG.width,"press c to jump");
			instructions2.size = instructions.size;
			instructions2.color = instructions.color;
			add(instructions2);
			
			restartDisplay = new FlxText(50,170,FlxG.width,"press r to try again");
			restartDisplay.size=16;
			restartDisplay.color=0xffeeeeee;
			restartDisplay.visible=false;
			add(restartDisplay);
			
			lengthDisplay = new FlxText(150,170,FlxG.width);
			lengthDisplay.size=16;
			lengthDisplay.alignment="center";
			lengthDisplay.visible=false;
			lengthDisplay.color=0xff222222;
			add(lengthDisplay);
			
			pole = new FlxSprite(polePosition,heightAfter-192);
			pole.loadGraphic(ImgPole,true,false,207,192);
			pole.addAnimation("0",[0]);
			pole.addAnimation("1",[1]);
			pole.addAnimation("2",[2]);
			pole.addAnimation("3",[3]);
			pole.addAnimation("4",[4]);
			pole.addAnimation("5",[5]);
			pole.addAnimation("6",[6]);
			pole.addAnimation("7",[7]);
			pole.addAnimation("8",[8]);
			pole.addAnimation("9",[9]);
			pole.addAnimation("10",[10]);
			pole.addAnimation("11",[11]);
			pole.addAnimation("12",[12]);
			pole.addAnimation("13",[13]);
			pole.play("0");
			pole.visible=false;
			add(pole);
			
			player = new FlxSprite(playerStartL,heightBefore-playerHeight);
			player.loadGraphic(ImgPlayer,true,false,16,16);
			player.addAnimation("standing",[0]);
			player.addAnimation("walking",[1,2,3],3);
			player.addAnimation("running",[4,5],6);
			player.addAnimation("jumping",[6]);
			player.addAnimation("climbing",[7,8],10,false);
			player.play("standing");
			
			player.drag.x=200;
			add(player);
		}
		public var _time:Number;
		
		override public function update():void
		{
			if (FlxG.keys.justPressed("R"))
			{
				FlxG.state = new GameState();
			}
			
			switch (playerstate)
			{
				case 0: //running
					if (FlxG.keys.justPressed("X"))
					{
						if (player._curAnim.name=="standing")
						{
							player.play("walking");
							FlxG.play(SndWalk);
						}
						else if (player._curAnim.name=="walking")
						{
							if (player.velocity.x>30)
							{
								player.play("running");	
//								FlxG.pause(SndWalk); kill the walking sound
								FlxG.play(SndRun);
							}
						}
						player.acceleration.x+=4;
					}
					if (FlxG.keys.justPressed("C"))
					{
						player.velocity.y=-100;
						playerstate=1;
						player.play("jumping");
						FlxG.play(SndJump, 1, false);
						//kill running/walking sound
					}
					if (player.x>dropPosition)
					{
						playerstate=6;
					}
					break;
				case 1: // jumping
				{
					player.velocity.y+=200*FlxG.elapsed;
					// have i hit the pole?
					if (player.x>polePosition)
					{
						this.rotspd=player.velocity.x * 1.2;
						pos = player.y-heightAfter;
						playerstate=2;
						//kill jumping sound
						player.play("climbing");
						FlxG.play(SndClimb,1,false);
					}
					
					// have I landed on the ground again (to the left of the pole)
					if (player.y > heightBefore - playerHeight )
					{
						// go back to the running state
						playerstate=0;
						//						player.velocity.x = 0;
						player.velocity.y = 0;
						player.y=heightBefore-playerHeight;
						player.play("walking");
						
					}
				}
					break;
				case 2:	// arcing over/climbing
				{
					if (FlxG.keys.justPressed("X"))
					{
						if(pos > -240){
							pos -= 5;
						}
						player.play("climbing");
					}
					angle += rotspd * FlxG.elapsed / 200.0;
					var frame:int = Math.max(Math.min(1.5*angle*14/(90.0*3.1415926535/180.0),13),0);
					this.pole.play(frame.toString());
					
					player.x = polePosition + pos*Math.sin(-angle);
					player.y = heightAfter + pos*Math.cos(-angle);
					
					if (FlxG.keys.justPressed("C"))
					{
						playerstate = 3;
						player.play("jumping");
						//kill climbing sound
						FlxG.play(SndJump, 1, false);
						// when you jump off, give player appropriate new velocity
						player.velocity.x=-rotspd *Math.sin(angle)*pos /(60.0*5)*2;
						player.velocity.y=rotspd *Math.cos(angle)* pos /(60.0*5);
					}	
					
					if (player.y>heightAfter-playerHeight)
					{
						player.y=heightAfter-playerHeight;
						player.velocity.x=0;
						player.velocity.y=0;
						player.acceleration.x=0;
						player.acceleration.y=0;
						playerstate=7;
						
						var distTravelled:Number = (player.x - polePosition)/4.0;
						lengthDisplay.visible=true;
						if(player.x < 374){
							lengthDisplay.text = "you didn't touch dirt";
						}else{
							lengthDisplay.text="you jumped " + distTravelled.toPrecision(4).toString() + " m!";
						}
						this.restartDisplay.visible = true;
					}
				}
					break;
				case 3: // jumping off
				{
					player.velocity.y+=200*FlxG.elapsed;
//					player.velocity.x *=2;
					if(angle < Math.PI /2){
						trace(angle);
						angle += rotspd * FlxG.elapsed / 200.0;
					}
					if (player.y+playerHeight>heightAfter)
					{
						player.y=heightAfter-playerHeight;
						player.velocity.x=0;
						player.velocity.y=0;
						player.acceleration.x=0;
						player.acceleration.y=0;
						playerstate = 7;
						
						distTravelled = (player.x - polePosition)/4.0;
						lengthDisplay.visible=true;
						if(player.x < 374){
							lengthDisplay.text = "you didn't touch dirt";
						}else{
							lengthDisplay.text="you jumped " + distTravelled.toPrecision(4).toString() + " m!";
						}
						this.restartDisplay.visible=true;
					}
				}
					break;
				case 4: // ending state - jump and land before pole
				{
					player.y=heightAfter-playerHeight;
					player.velocity.x=0;
					player.velocity.y=0;
					player.acceleration.x=0;
					player.acceleration.y=0;
					
					lengthDisplay.visible=true;
					lengthDisplay.text="You jumped too soon and missed the pole!";
					this.restartDisplay.visible=true;
				}
					break;
				case 5: // ending state - jump on pole with insufficient velocity
				{
					player.y=heightAfter-playerHeight;
					player.velocity.x=0;
					player.velocity.y=0;
					player.acceleration.x=0;
					player.acceleration.y=0;
					lengthDisplay.visible=true;
					lengthDisplay.text="Too slow - the pole won't budge!";
					this.restartDisplay.visible=true;
				}	
					break;
				case 6: // ending state - don't jump
				{
					player.y=heightAfter-playerHeight;
					player.velocity.x=0;
					player.velocity.y=0;
					player.acceleration.x=0;
					player.acceleration.y=0;
					lengthDisplay.visible=true;
					lengthDisplay.text="You forgot to jump.";
					player.play("standing");
					this.restartDisplay.visible=true;
				}
					break;
				case 7: // ending state - success
				{
					player.y=heightAfter-playerHeight;
					player.velocity.x=0;
					player.velocity.y=0;
					player.acceleration.x=0;
					player.acceleration.y=0;
				}
					break;
			}
			_time+=FlxG.elapsed;
			super.update();
		}
		override public function postProcess():void
		{
			var p:Point = new Point(0, 0);
			var s:Shape = new Shape();
			s.graphics.lineStyle(3, 0x111111);
			s.graphics.moveTo(polePosition+16, heightAfter);
			s.graphics.lineTo((polePosition+16 - 240*Math.sin(-angle)), heightAfter - 240*Math.cos(-angle));
			FlxG.buffer.draw(s);		
			player.render();
		}
	}
}